@@ -37,7 +37,8 @@ from scipy.ndimage import zoom
 from scipy.spatial import KDTree
 from scipy.interpolate import interp1d
 from matplotlib.colors import LogNorm
-from photutils import DAOStarFinder, CircularAperture
+from photutils.detection import DAOStarFinder
+from photutils.aperture import CircularAperture
 from photutils.background import Background2D, MedianBackground
 from astropy.io import fits
 from astropy.convolution import Gaussian2DKernel, convolve
@@ -128,11 +129,9 @@ shift_algorithm = 'multiple_star' # 'single_star' or 'multiple_star'.
 
 min_exptime = 100 # will ignore orbits with exptimes below limit.
 
-'''Astroalign settings. Change only if you know what you are 
-doing.'''
-aa.NUM_NEAREST_NEIGHBORS = 7
-aa.MIN_MATCHES_FRACTION = 0.1
-aa.PIXEL_TOL = 2
+# Astroalign defaults. 
+NUM_NEAREST_NEIGHBORS = 7
+MIN_MATCHES_FRACTION = 0.1
 
 # For Astrometry.
 AstrometryNet_API_key = 'ujmrvwqqyelxmzcj'  
@@ -1116,7 +1115,8 @@ def process_ccdlab(output = None,
 def make_image(events_list = events_list, 
                framecount_per_sec = framecount_per_sec):
 
-    """Create a FITS image from the input events list.
+    """Create a FITS image from the input events list. 
+    The image unit is in counts. 
 
     Parameters
     ----------
@@ -1235,6 +1235,9 @@ def new_detect_sources_daofind(fx, fy, photons, threshold, framecount_per_sec):
     return uA
 
 def combine_events_lists(events_lists_paths = None, 
+                         threshold = 5,
+                         NUM_NEAREST_NEIGHBORS = NUM_NEAREST_NEIGHBORS,
+                         MIN_MATCHES_FRACTION = MIN_MATCHES_FRACTION,
                          shift_algorithm = shift_algorithm, 
                          min_exptime = min_exptime,
                          framecount_per_sec = framecount_per_sec):
@@ -1246,14 +1249,30 @@ def combine_events_lists(events_lists_paths = None,
     events_lists_paths : list
         The list of events list FITS file paths.
         
+    threshold : float, optional
+        The threshold parameter associated with the source detection method. 
+        The default value is 5. The threshold is lowered in steps of 0.5
+        until a minimum number of sources are detected. The minimum number
+        depends on the **shift_algorithm**.  
+        
+    NUM_NEAREST_NEIGHBORS : int, optional    
+        See https://astroalign.quatrope.org/en/latest/api.html#astroalign.NUM_NEAREST_NEIGHBORS
+        for details. The default value is 7. Only relevant for ``'multiple_star'`` method.
+
+    MIN_MATCHES_FRACTION : float, optional    
+        See https://astroalign.quatrope.org/en/latest/api.html#astroalign.MIN_MATCHES_FRACTION
+        for details. The default value is 0.1. Only relevant for ``'multiple_star'`` method.
+                        
     shift_algorithm : {'single_star', 'multiple_star'}, optional
         The parameter to choose between available aligning methods. 
         
         * ``'single_star'``: To use a single star for aligning orbits. 
             Single_star option is useful only when there is only one star 
-            in the field and no rotation between frames.
+            in the field and no rotation between frames. The minimum number of
+            source detection is 2. 
         * ``'multiple_star'``: To use multiple stars for aligning orbits. 
-            This is recommended and the default method.
+            This is the **recommended** and default method. The minimum 
+            number of source detection is 3. 
                                
     min_exptime : float, optional
         Orbits having exposure time below this limit will be ignored. 
@@ -1278,6 +1297,14 @@ def combine_events_lists(events_lists_paths = None,
     >>> file_path_list = glob('*/*/*/RAS_VIS/*/F*/*F1*ce*')
     >>> curvit.combine_events_lists(file_path_list)
     """
+
+    aa.NUM_NEAREST_NEIGHBORS = NUM_NEAREST_NEIGHBORS
+    aa.MIN_MATCHES_FRACTION = MIN_MATCHES_FRACTION
+    aa.PIXEL_TOL = 2    
+        
+    if shift_algorithm not in ['single_star', 'multiple_star']:
+        print('\nInvalid input for "shift_algorithm" parameter.\n')
+        return   
     
     print('\nWorking on the following events lists:')
     print(*events_lists_paths, sep='\n')
@@ -1296,7 +1323,7 @@ def combine_events_lists(events_lists_paths = None,
         uA = []    
         if shift_algorithm == 'single_star':
             number_of_sources = 2
-            i = 6
+            i = threshold
             while len(uA) <= number_of_sources:
                 uA = new_detect_sources_daofind(fx, fy, photons, i, framecount_per_sec)
                 i = i - 0.5
@@ -1305,7 +1332,7 @@ def combine_events_lists(events_lists_paths = None,
                     break
         else:
             number_of_sources = 3
-            i = 5
+            i = threshold
             while len(uA) <= number_of_sources:
                 uA = new_detect_sources_daofind(fx, fy, photons, i, framecount_per_sec)
                 if len(uA) > 200:
@@ -1488,7 +1515,7 @@ def combine_events_lists(events_lists_paths = None,
     exp_time = unique_frames / AVGFRMRT 
     print('Total combined exposure time = {:.4f} seconds'.format(exp_time))
     hdu_base[0].header['EXPTIME'] = exp_time
-    hdu_base.flush()
+    hdu_base.close()
     print('The combined events list: {}'.format(combined_eventslist_name))    
     print("\nDone!\n")
     
@@ -1524,7 +1551,7 @@ def image_astrometry(UV_image = None, threshold = 3, API_key = AstrometryNet_API
     >>> import curvit
     >>> curvit.image_astrometry('test.fits')        
     """    
-    hdu = fits.open(UV_image)
+    hdu = fits.open(UV_image, mode = 'update')
     sources = daofind_on_image_data(hdu[0].data, threshold)
     sources = sources + 1
     print('Number of detected sources = {}'.format(len(sources)))
@@ -1586,7 +1613,8 @@ def image_astrometry(UV_image = None, threshold = 3, API_key = AstrometryNet_API
             pass 
             
         hdu[0].header.update(wcs_header)
-        hdu.writeto(UV_image, overwrite = True)
         print('Image header updated with WCS.')
     else:
-        print('\nAstrometry.net solve FAILED!!!\n')        
+        print('\nAstrometry.net solve FAILED!!!\n')  
+    # To save all changes.    
+    hdu.close()        
